@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2019,2020 Thomas E. Dickey                                     *
+ * Copyright 2019-2021,2022 Thomas E. Dickey                                *
  * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -52,7 +52,7 @@
  * scroll operation worked, and the refresh() code only had to do a
  * partial repaint.
  *
- * $Id: view.c,v 1.138 2020/02/02 23:34:34 tom Exp $
+ * $Id: view.c,v 1.145 2022/12/04 00:40:11 tom Exp $
  */
 
 #include <test.priv.h>
@@ -62,7 +62,7 @@
 #include <sys/stat.h>
 #include <time.h>
 
-static void finish(int sig) GCC_NORETURN;
+static GCC_NORETURN void finish(int sig);
 
 #define my_pair 1
 
@@ -78,9 +78,7 @@ static int num_lines;
 static bool n_option = FALSE;
 #endif
 
-static void usage(void) GCC_NORETURN;
-
-static void
+static GCC_NORETURN void
 failed(const char *msg)
 {
     endwin();
@@ -92,12 +90,10 @@ static int
 ch_len(NCURSES_CH_T *src)
 {
     int result = 0;
-#if USE_WIDEC_SUPPORT
-    int count;
-#endif
 
 #if USE_WIDEC_SUPPORT
     for (;;) {
+	int count;
 	TEST_CCHAR(src, count, {
 	    int len = wcwidth(test_wch[0]);
 	    result += (len > 0) ? len : 1;
@@ -136,7 +132,6 @@ show_all(const char *tag)
     int i;
     int digits;
     char temp[BUFSIZ];
-    NCURSES_CH_T *s;
     time_t this_time;
 
     for (digits = 1, i = num_lines; i > 0; i /= 10) {
@@ -161,8 +156,10 @@ show_all(const char *tag)
 
     scrollok(stdscr, FALSE);	/* prevent screen from moving */
     for (i = 1; i < LINES; i++) {
+	NCURSES_CH_T *s;
 	int len;
 	int actual = (int) (lptr + i - vec_lines);
+
 	if (actual > num_lines) {
 	    if (i < LINES - 1) {
 		int y, x;
@@ -189,8 +186,11 @@ show_all(const char *tag)
 	     */
 	    {
 		int j;
-		int width = 1, count;
+		int width = 1;
+
 		for (j = actual = 0; j < shift; ++j) {
+		    int count;
+
 		    TEST_CCHAR(s + j, count, {
 			width = wcwidth(test_wch[0]);
 		    }
@@ -257,9 +257,12 @@ read_file(const char *filename)
     }
 
     len = fread(my_blob, sizeof(char), (size_t) sb.st_size, fp);
-    my_blob[sb.st_size] = '\0';
     fclose(fp);
 
+    if (len > (size_t) sb.st_size)
+	len = (size_t) sb.st_size;
+    my_blob[len] = '\0';
+
     for (pass = 0; pass < 2; ++pass) {
 	char *base = my_blob;
 	k = 0;
@@ -273,12 +276,19 @@ read_file(const char *filename)
 		++k;
 	    }
 	}
+	if (base != (my_blob + j)) {
+	    if (pass)
+		my_vec[k] = base;
+	    ++k;
+	}
 	num_lines = k;
-	if (base != (my_blob + j))
-	    ++num_lines;
-	if (!pass &&
-	    ((my_vec = typeCalloc(char *, (size_t) k + 2)) == 0)) {
-	    failed("cannot allocate line-vector #1");
+	if (pass == 0) {
+	    if (((my_vec = typeCalloc(char *, (size_t) k + 2)) == 0)) {
+		failed("cannot allocate line-vector #1");
+	    }
+	} else {
+	    if (my_vec[0] == NULL)
+		my_vec[0] = my_blob;
 	}
     }
 
@@ -366,13 +376,14 @@ read_file(const char *filename)
     free(my_blob);
 }
 
-static void
-usage(void)
+static GCC_NORETURN void
+usage(int ok)
 {
     static const char *msg[] =
     {
 	"Usage: view [options] file"
 	,""
+	,USAGE_COMMON
 	,"Options:"
 	," -c       use color if terminal supports it"
 	," -i       ignore INT, QUIT, TERM signals"
@@ -388,8 +399,11 @@ usage(void)
     size_t n;
     for (n = 0; n < SIZEOF(msg); n++)
 	fprintf(stderr, "%s\n", msg[n]);
-    ExitProgram(EXIT_FAILURE);
+    ExitProgram(ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
+/* *INDENT-OFF* */
+VERSION_COMMON()
+/* *INDENT-ON* */
 
 int
 main(int argc, char *argv[])
@@ -419,6 +433,7 @@ main(int argc, char *argv[])
 	0
     };
 
+    int ch;
     int i;
     int my_delay = 0;
     NCURSES_CH_T **olptr;
@@ -431,8 +446,8 @@ main(int argc, char *argv[])
 
     setlocale(LC_ALL, "");
 
-    while ((i = getopt(argc, argv, "cinstT:")) != -1) {
-	switch (i) {
+    while ((ch = getopt(argc, argv, OPTS_COMMON "cinstT:")) != -1) {
+	switch (ch) {
 	case 'c':
 	    try_color = TRUE;
 	    break;
@@ -453,7 +468,7 @@ main(int argc, char *argv[])
 		char *next = 0;
 		int tvalue = (int) strtol(optarg, &next, 0);
 		if (tvalue < 0 || (next != 0 && *next != 0))
-		    usage();
+		    usage(FALSE);
 		curses_trace((unsigned) tvalue);
 	    }
 	    break;
@@ -461,12 +476,16 @@ main(int argc, char *argv[])
 	    curses_trace(TRACE_CALLS);
 	    break;
 #endif
+	case OPTS_VERSION:
+	    show_version(argv);
+	    ExitProgram(EXIT_SUCCESS);
 	default:
-	    usage();
+	    usage(ch == OPTS_USAGE);
+	    /* NOTREACHED */
 	}
     }
     if (optind + 1 != argc)
-	usage();
+	usage(FALSE);
 
     InitAndCatch(initscr(), ignore_sigs ? SIG_IGN : finish);
     keypad(stdscr, TRUE);	/* enable keyboard mapping */
